import os,sys
root_path = os.path.abspath(os.path.join(os.path.dirname(__file__),'..'))
if root_path not in sys.path:
    sys.path.append(root_path)


import argparse

parser = argparse.ArgumentParser(prog='color-theme-analyse',formatter_class=argparse.RawDescriptionHelpFormatter, description='功能介绍', epilog='基于 MMCQ 对图片进行色彩主题分析，采用图片压缩和多进程来加速批量分析速度。\n\n https://gitee.com/hi-windom/color-theme-analyse  \n \n ')
parser.add_argument('--path',default=None, help='选择交给 %(prog)s 处理的文件夹（包含子文件夹）')

args = parser.parse_args()


if __name__ == '__main__':
    SRC_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__),'src'))
    try:
        from MMCQsc.scp import executable_check
    except:
        exit()
    else:
        from MMCQsc.scp import main
        try:
            result = main.mainFunc()
        finally:
            exit()



# -*- coding=utf-8

# SCSD-PY001

# hi-windom/ColorThemeAnalyse

# https://gitee.com/hi-windom/color-theme-analyse

'''
# ---------------------------------
# 创建于    2021-7-20
# 更新于    2021-7-20 02:23:27
# ---------------------------------
# Need help ?  => 694357845@qq.com
# ---------------------------------
#  作者很懒，还没想好说些什么
# ---------------------------------
'''

################################################################
import logging

BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = range(8)
RESET_SEQ = "\033[0m"
COLOR_SEQ = "\033[1;%dm"

COLORS = {
    'WARNING': GREEN,
    'INFO': CYAN,
    'DEBUG': WHITE,
    'CRITICAL': YELLOW,
    'ERROR': RED,
}
""" logger.debug('debug 信息 蓝色')
    logger.info('info 信息 白色')
    logger.warning('warning 信息 绿色')
    logger.error('error 信息 红色')
    logger.critical('critial 信息 橙色')  // alias for fatal error  """

class ColoredFormatter(logging.Formatter):
    def __init__(self, msg, use_color=True):
        logging.Formatter.__init__(self, msg)
        self.use_color = use_color

    def format(self, record):
        levelname = record.levelname
        message = str(record.msg)
        funcName = record.funcName
        if self.use_color and levelname in COLORS:
            levelname_color = COLOR_SEQ % (30 + COLORS[levelname]) + levelname + RESET_SEQ
            message_color = COLOR_SEQ % (30 + COLORS[levelname]) + message + RESET_SEQ
            funcName_color = COLOR_SEQ % (30 + COLORS[levelname]) + funcName + RESET_SEQ
            record.levelname = levelname_color
            record.msg = message_color
            record.funcName = funcName_color
        return logging.Formatter.format(self, record)


LOGFORMAT = " >>> %(filename)s:%(lineno)d [ %(levelname)s ]\t%(asctime)s    %(message)s\t%(name)s"
LOG_LEVEL = logging.DEBUG
formatter = ColoredFormatter(LOGFORMAT)
stream = logging.StreamHandler()
stream.setLevel(LOG_LEVEL)
stream.setFormatter(formatter)
logging.root.setLevel(LOG_LEVEL)
log = logging.getLogger('logconfig')
log.setLevel(LOG_LEVEL)
log.addHandler(stream)

def myLogging(name):
    log = logging.getLogger(name)
    log.setLevel(LOG_LEVEL)
    log.addHandler(stream)
    return log



# -*- coding=utf-8

# SCSD-PY001

# hi-windom/ColorThemeAnalyse

# https://gitee.com/hi-windom/color-theme-analyse

'''
# ---------------------------------
# 创建于    2021-5-18
# 更新于    2021-7-20 02:10:10
# ---------------------------------
# Need help ?  => 694357845@qq.com
# ---------------------------------
# 处理流程：
# 1.传入路径
# 2.复制路径文件夹的图片 => 粘贴至当前PY文件目录下的prepare文件夹（没有会自动创建）
# 3.压缩prepare文件夹的图片 => 另存至当前PY文件目录下的compress文件夹（没有会自动创建） //这里使用率多进程+多线程，并绑定回调函数testMMCQ执行4.
# 4.分析compress文件夹的图片色彩主题 => 重命名对应的prepare图片文件
# 5.对prepare文件夹的图片生成对应的JSON文件
# 6.[Optional]生成报告
# ---------------------------------
'''
import os,sys
import json
from importlib import import_module
root_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "../.."))
BASE_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "../.."))
if root_path not in sys.path:
    sys.path.append(root_path)
from MMCQsc.scp.lib import logger
logger = logger.myLogging("gitee.com/soltus")
# 全局变量
try:
    profile = json.load(open(f'{root_path}\\scp\\scripts\\profile.json', 'r+'))
    themes = profile['themes']
    size_rate = profile['size_rate']
    ignore_size = profile['ignore_size']
except:
    logger.error('./profile.json 配置文件加载失败')

'''
# themes 是分析色彩的结果颜色数量，默认为5
# size_rate 是图片尺寸缩放倍率，默认为10
# ignore_size 是图片压缩忽略阈值（宽/高中的最大值），默认为300
# 推荐在profile.json中修改配置，如有需要也可以修改成传参
'''

from concurrent import futures  # 多进程+多线程，实测速度提升明显，缺点是无法实现rich进度条展示
from multiprocessing import shared_memory
from posixpath import pathsep
import shutil
import datetime
import time  # 导入EasyGui模块，主要用于选择目标文件夹，不需要可以去掉
import urllib.parse as parse
import binascii
import re
from hashlib import md5
# try:
#     import easygui as g
#     import cv2 as cv
#     import numpy as np
#     from PIL import Image
#     from rich import print  # rich用于进度条展示和美化终端输出，不需要可以去掉
#     from rich.console import Console
#     from rich.progress import (
#     BarColumn,
#     Progress,
# )
# except ImportError:
#     raise
class Pgd:
    def __init__(self):
        self.url = 'https://pypi.douban.com/simple/'

    def task(self,im,re):
        self.im = im
        self.re = re
        import os
        repo = input('\n\n Unable to import package [{}] , \n\n Do you want to download ? \n\n\t\tProccess ? [Y/n]\t'.format(self.im))
        if repo in ['Y','y']:
            os.system('cls')
            os.system("pip install {} -i {}".format(self.re, self.url))
            return 1
        return 0

dddd = 0
pgd = Pgd()
try:
    import easygui as g
except ImportError:
    repo = pgd.task(im="easygui",re="easygui")
    dddd += repo

try:
    import cv2 as cv
except ImportError:
    repo = pgd.task(im="cv2",re="opencv-python")
    dddd += repo

try:
    import numpy as np
except ImportError:
    repo = pgd.task(im="numpy",re="numpy")
    dddd += repo

try:
    from PIL import Image
except ImportError:
    repo = pgd.task(im="PIL",re="pillow")
    dddd += repo

try:
    from rich import print
    from rich.console import Console
    from rich.progress import (
    BarColumn,
    Progress,
)
except ImportError:
    repo = pgd.task(im="rich",re="rich")
    dddd += repo


if dddd:
    os.system("cls")
    print(f'\n\t\t{dddd} new packages already installed .\n\n\t\ttry to launch again .\n\n')
    exit()


from MMCQsc.scp.lib.MMCQ import MMCQ # 第一个MMCQ是文件名，第二个是类名

    # from ..lib.logger import *
    # logger = myLogging("gitee.com/soltus")
    # from ..lib.MMCQ import MMCQ  # 第一个MMCQ是文件名，第二个是类名


# 实例化进度条，由于采用多进程+多线程，只能当分隔符使用
progress = Progress(
    BarColumn(bar_width=None)
)

console = Console(color_system='auto', style=None)


def procompress(files, root):
    testshm = shared_memory.SharedMemory(name='main_run_share')
    buf = testshm.buf
    buf[3] += 1
    console.rule(title=' 多进程 ProcessPoolExecutor {:<3}'.format(
                 str(buf[3])), align='center')

    ss = []
    ipl = 0
    for f in files:
        if os.path.splitext(f)[1].lower() in ['.jpg', 'jpeg', '.png']:
            ipl = ipl + 1
            new_file_path = r'%s\%s_%s_%s%s' % (
                os.path.join(BASE_DIR, "src\\prepare"), 'img', ipl, str(int(time.time()*10000)), os.path.splitext(f)[1])
            shutil.copy2(os.path.join(root, f), new_file_path)
            ss.append(new_file_path)

    with futures.ThreadPoolExecutor(max_workers=None) as pool:  # 多线程
        for si in ss:
            results = pool.submit(compressImage, si)
            results.add_done_callback(testMMCQ)  # 回调函数


def compressImage(srcPath):
    # 如果是文件就处理
    if os.path.isfile(srcPath):
        try:
            # 打开原图片缩小后保存，可以用if srcFile.endswith(".jpg")或者split，splitext等函数等针对特定文件压缩
            filename = os.path.basename(srcPath)
            srcFile = srcPath
            dstFile = os.path.join(os.path.join(
                BASE_DIR, "src\\compress"), filename)
            sImg = Image.open(srcFile)
            dImg = sImg.convert('RGB')
            w, h = sImg.size
            MIN_SIZE = 40*40
            # 设置压缩尺寸和选项，注意尺寸要用括号
            if max(w, h) > ignore_size:
                dImg = dImg.resize(
                    (int(w/size_rate), int(h/size_rate)), Image.NEAREST)
            # 尺寸过小需要放大
            elif max(w, h) < MIN_SIZE:
                dImg = dImg.resize(
                    (int(w*2), int(h/size_rate*2)), Image.NEAREST)
            dImg.save(dstFile.replace(filename.split('.')[1], 'jpg'))
            dImg.close()
            sImg.close()
            os.rename(srcFile, srcFile.replace(
                filename.split('.')[1], 'jpg'))  # 不改找不到文件
            return dstFile.replace(filename.split('.')[1], 'jpg')

        except Exception:
            sImg.close()
            sImg = cv.imread(srcFile, 1)
            if max(w, h) > ignore_size:
                dImg = cv.resize(sImg, None, size_rate, size_rate)
            cv.imwrite(dstFile.replace(filename.split('.')[1], 'jpg'), dImg)
            sImg.close
            os.rename(srcFile, srcFile.replace(
                filename.split('.')[1], 'jpg'))  # 不改找不到文件
            return dstFile.replace(filename.split('.')[1], 'jpg')

    # 如果是文件夹就递归
    # if os.path.isdir(srcPath):
    #     for file in srcPath:
    #         compressImage(file)
    # 多进程已做好文件夹递归，传参必是文件


def testMMCQ(future):
    testshm = shared_memory.SharedMemory(name='main_run_share')
    buf = testshm.buf
    if buf[4] < 255:
        buf[4] += 1
    elif buf[5] < 255:
        buf[5] += 1
    elif buf[6] < 255:
        buf[6] += 1
    elif buf[7] < 255:
        buf[7] += 1  # 处理不超过255*4=1020张图片
    thisbuf = str(buf[4] + buf[5] + buf[6] + buf[7])

    imgfile = future.result()
    rgb = list(map(lambda d: MMCQ(d, themes, file=imgfile).quantize(), [cv.imdecode(np.fromfile(
        imgfile, dtype=np.uint8), cv.COLOR_BGR2RGB)]))
    for i in range(len(rgb)):
        strjoin = ''
        for j in range(len(rgb[i])):
            strs = '#'
            for k in rgb[i][j]:
                strs += str(hex(k))[-2:].replace('x', '0').upper()
            strjoin += strs
        console.rule(title='多线程 ThreadPoolExecutor {:<9}  结果 {}\n'.format(
            thisbuf, strjoin.replace('#', ' ')), align='left')
        strjoin += '__{}__'.format(thisbuf + str(int(time.time())))
        filename = os.path.basename(imgfile)
        extname = os.path.splitext(imgfile)[1]
        newname = imgfile.replace(
            'compress', 'prepare').replace(filename, strjoin)
        os.rename(imgfile.replace(
            'compress', 'prepare'), newname + extname)


def domain(img):
    testshm = shared_memory.SharedMemory(name='main_run_share')
    buf = testshm.buf
    with progress:
        task_id = progress.add_task(
            "process", filename="正在多线程分析 ", start=False)

        totaltime = time.time()  # 获取当前文件目录

        # 遍历删除图片
        path = os.path.join(BASE_DIR, "src\\finish")
        if os.path.exists(path):
            shutil.rmtree(path)
        os.mkdir(path)
        path = os.path.join(BASE_DIR, "src\\compress")
        if os.path.exists(path):
            shutil.rmtree(path)
        os.mkdir(path)
        if os.path.exists(os.path.join(BASE_DIR, "src\\reports")) == False:
            os.makedirs(os.path.join(BASE_DIR, "src\\reports"))
        path = os.path.join(BASE_DIR, "src\\prepare\\temp")
        temp = os.path.join(BASE_DIR, "src\\prepare\\temp")
        if os.path.exists(path):
            shutil.rmtree(path)
        os.makedirs(path)
        path = os.walk(os.path.join(BASE_DIR, "src\\prepare"))
        rerule = re.compile(r'\#.{6}\#.{6}\#.{6}\#.{6}\#.{6}\__')
        rerule2 = re.compile(
            r'SCMD-P.*')  # 和定义的命名规则有关
        redoma = 0
        for root, dirs, files in path:
            for f in files:
                if rerule.search(f) or rerule2.search(f):
                    os.remove(os.path.join(root, f))
                else:
                    if root != temp:
                        redoma += 1
                        shutil.move(os.path.join(root, f), temp)
        if redoma > 1:
            progress.update(task_id, visible=False, refresh=False)
            progress.stop()
            os.system('cls')
            redo = input(
                '\n\n\n注意：检测到遗留的 {} 个任务未处理，是否先继续未完成的任务？\n\n  __ 输入 y 继续遗留任务，输入 n 放弃遗留任务，任意其他键退出 __\n'.format(redoma))
            if redo in ['y', 'Y']:
                procompress(os.listdir(temp), temp)
                shutil.rmtree(temp)
            elif redo in ['n', 'N']:
                shutil.rmtree(temp)
                progress.update(task_id, visible=True, refresh=True)
                progress.start_task(task_id)
            else:
                exit()

        if img != None:  # 有传入才处理

            ptv = 0
            origin_list = []
            path = os.walk(os.path.dirname(img))
            for root, dirs, files in path:
                for f in files:
                    if os.path.splitext(f)[1].lower() in ['.jpg', '.jpeg', '.png']:
                        ptv += 1
                        origin_list.append(os.path.join(root, f))
            progress.update(task_id, total=ptv)
            progress.start_task(task_id)
            progress.update(task_id, advance=ptv)
            if ptv > 1019:
                console.rule(title='demo限制单次最多处理 1020 张图片（ {} 张已选择）'.format(
                    ptv), align='center')
                exit()

            # 开始处理

            with futures.ProcessPoolExecutor(max_workers=None) as prolist:
                ffs = []
                roots = []
                path = os.walk(os.path.dirname(img))
                for root, dirs, files in path:
                    ffs.append(files)
                    roots.append(root)

                prolist.map(procompress, ffs, roots)

            while True:  # 定时检测文件夹文件数量
                nowlist = len(os.listdir(
                    os.path.join(BASE_DIR, "src\\finish")))
                time.sleep(2)
                while nowlist != len(os.listdir(
                        os.path.join(BASE_DIR, "src\\finish"))):
                    continue
                buf[1] = 0
                break

            path = os.walk(os.path.join(BASE_DIR, "src\\compress"))
            for root, dirs, files in path:
                for f in files:
                    shutil.move(os.path.join(root, f), os.path.join(
                        BASE_DIR, "src\\finish", f))

            path = os.walk(os.path.join(BASE_DIR, "src\\prepare"))
            for root, dirs, files in path:
                for f in files:
                    fname = os.path.splitext(os.path.basename(f))[0]
                    fnames = fname.split('\\')
                    lastname = fnames[len(fnames) - 1]
                    fname = fname.replace(
                        lastname, re.sub(r'\__.*\__', '', lastname))  # 正则替换

                    curr_time = datetime.datetime.now()
                    with open(os.path.join(root, f), mode="rb") as bf:
                        crc32v = hex(binascii.crc32(bf.read()))[2:].upper()
                        bf.seek(0)
                        md5v = md5(bf.read()).hexdigest()
                    licenseid = "SCMD-P_L1F_" + curr_time.strftime("%Y%m") + \
                        fname.replace("#", "-")
                    jsonstr = {'license': licenseid, 'color': fname, 'theme': fname.split("#")[1:], 'CRC32': crc32v, 'size': os.path.getsize(
                        os.path.join(root, f)), 'date': curr_time.strftime("%Y-%m-%d"), 'time': curr_time.strftime("%H:%M:%S"), 'md5': md5v}
                    dstname = os.path.join(
                        root, licenseid) + os.path.splitext(f)[1]
                    if os.path.exists(dstname):
                        sames = [dstname, os.path.join(root, f), os.path.getsize(
                            dstname), os.path.getsize(os.path.join(root, f))]  # 第二个是当前处理对象
                        issanme = g.buttonbox(msg=f'\n 【疑似重复提醒】 检测到两张图片的色彩主题相同，你有理由怀疑它们是重复的，请检查！\n{sames[0]} \
                            为已存在图片（图片大小{sames[2]}）\n{sames[1]} 为当前待保存图片（图片大小{sames[3]}）\n',
                                              title=' 操作确认 ', choices=(
                                                  ' 保留已存在的 ', ' 我全都要 ', ' 删除已存在的 '), default_choice=None, cancel_choice=' 我全都要 ', images=None)
                        if issanme != ' 我全都要 ':
                            if issanme == ' 保留已存在的 ':
                                os.remove(dstname)
                                with open(os.path.join(root, licenseid) + '.json', 'w') as js:
                                    json.dump(jsonstr, js)
                                os.rename(os.path.join(root, f), dstname)
                            else:
                                os.remove(os.path.join(root, f))
                        else:
                            # 本demo参考SCMD标准，因此不允许重复存在，如有需求可自行修改
                            os.remove(os.path.join(root, f))
                    else:
                        with open(os.path.join(root, licenseid) + '.json', 'w') as js:
                            json.dump(jsonstr, js)
                        os.rename(os.path.join(root, f), dstname)

            progress.stop_task(task_id)

            cost_times = str(round(time.time()-totaltime, 4))
            cc = g.ccbox(msg="\n\n\n" + BASE_DIR + "\n\n\n 总耗时：" +
                         cost_times, title="处理完成", choices=(" 生成报告 ", " 完成 "))

            # 点击生成报告触发事件
            if cc == 1:
                cc1 = shared_memory.SharedMemory(name='main_run_share')
                buf = cc1.buf
                buf[2] = 1
                print('\n\n\n')
                curr_time = datetime.datetime.now()
                # 将路径转为URL格式，不转则json.dump以ASCII格式写到JSON文件里
                in_path_url = parse.quote(
                    os.path.dirname(img), safe=";/?:@&=+$,")
                reportjson = {'date': curr_time.strftime(
                    "%Y-%m-%d"), 'time': curr_time.strftime("%H:%M:%S"), 'in_path_url': in_path_url, 'in_files': ptv, 'cost_times': cost_times}
                reportfile = os.path.join(
                    BASE_DIR, 'src\\reports\\' + curr_time.strftime("%Y%m%d-%H%M%S_")) + 'report.json'
                with open(reportfile, 'w') as js:
                    file_list = []
                    for dir in os.listdir(BASE_DIR + '\\src\\prepare'):
                        child = os.path.join(BASE_DIR + '\\src\\prepare', dir)
                        if os.path.isdir(child):
                            for file in os.listdir(child):
                                if os.path.splitext(file)[1].lower() in ['.jpg', '.jpeg', '.png']:
                                    file = os.path.join(child, file)
                                    if os.path.basename(child) != 'temp':
                                        file_list.append(
                                            file.replace('\\', '/'))
                        elif os.path.isfile(child):
                            if os.path.splitext(child)[1].lower() in ['.jpg', '.jpeg', '.png']:
                                file_list.append(child.replace('\\', '/'))
                    reportjson['origin_list'] = origin_list
                    json.dump(reportjson, js)
                    console.print(reportjson, justify='full',
                                  highlight=True)
                    for i in range(len(file_list)):
                        file_list[i] = str(file_list[i]).replace(
                            BASE_DIR.replace('\\', '/') + '/src/', '')
                    with open(os.path.join(BASE_DIR, 'src/index.js'), 'w', encoding='utf-8') as mainjs:
                        mainjs.write(
                            'class ImgShow extends React.Component {\n render() {\n return (<div id="imgbox"><div className="imgshow1" >\n')
                    with open(os.path.join(BASE_DIR, 'src/index.js'), 'a', encoding='utf-8') as mainjs:
                        img1 = len(file_list) // 2
                        img2_w = False
                        i = 0
                        shutil.copy2(os.path.join(
                            BASE_DIR, 'src/_css/base.css'), os.path.join(BASE_DIR, 'src/index.css'))
                        for name in file_list:
                            i += 1
                            if rerule2.search(os.path.splitext(name)[0]):
                                colors = rerule2.search(os.path.splitext(name)[
                                    0]).group().split('-')
                            else:
                                colors = ['', '', 'FFFFFF', 'FFFFFF',
                                          'FFFFFF', 'FFFFFF', 'FFFFFF']  # 意外处理
                            if len(colors) != 7:
                                print(colors)
                                print('格式有误')
                            else:
                                if i <= img1 and img2_w == False:
                                    mainjs.writelines(
                                        r'<div className="colorbox wow fadeIn" data-wow-delay="0.2s" data-wow-duration="0.5s" data-wow-offset="20">')
                                    with open(os.path.join(BASE_DIR, 'src/index.css'), 'a', encoding='utf-8') as maincss:
                                        maincss.writelines('\n')
                                        for c in range(1, 6):
                                            mainjs.writelines(
                                                r'<div id = "t{}c{}"></div>'.format(i, c))
                                            maincss.writelines(
                                                '#t{}c{} {{\nmax-width: 20%; min-width: 20%; width: 20%; height: 40px; background-color: #{}; display: inline-block; z-index: 99;\n}}\n'.format(i, c, colors[c+1]))
                                    mainjs.writelines('</div>\n')
                                    mainjs.writelines(
                                        r'<img className="img wow fadeIn" data-wow-delay="0.2s" data-wow-duration="0.5s" data-wow-offset="20" src={"' + name + r'"} />' + '\n')
                                else:
                                    if img2_w == False:
                                        mainjs.writelines(
                                            '</div>\n<div className='"'imgshow2'"'>\n')
                                        img2_w = True
                                    mainjs.writelines(
                                        r'<div className="colorbox wow fadeIn" data-wow-delay="0.2s" data-wow-duration="0.5s" data-wow-offset="20">')
                                    with open(os.path.join(BASE_DIR, 'src/index.css'), 'a', encoding='utf-8') as maincss:
                                        maincss.writelines('\n')
                                        for c in range(1, 6):
                                            mainjs.writelines(
                                                r'<div id = "t{}c{}"></div>'.format(i, c))
                                            maincss.writelines(
                                                '#t{}c{} {{\nmax-width: 20%; min-width: 20%; width: 20%; height: 40px; background-color: #{}; display: inline-block; z-index: 99;\n}}\n'.format(i, c, colors[c+1]))
                                    mainjs.writelines('</div>\n')
                                if img2_w == True:
                                    mainjs.writelines(
                                        r'<img className="img wow fadeIn" data-wow-delay="0.2s" data-wow-duration="0.5s" data-wow-offset="20" src={"' + name + r'"} />' + '\n')
                        mainjs.writelines(
                            '</div>\n</div> ); \n}}\n\nReactDOM.render(<ImgShow / >,document.getElementById('"'imgs'"')); ')
                        with open(os.path.join(BASE_DIR, 'src/_js/base.js'), 'r+', encoding='utf-8') as index:
                            index.seek(0)
                            index_text = index.read()
                            rf = re.sub(r'^.*\\src', './',
                                        reportfile).replace('\\', '/')
                            index_text = index_text.replace(
                                '[[reportpath]]', rf)
                            mainjs.writelines(index_text)


# if __name__ == '__main__':
#     # fileopenbox()函数的返回值是你选择的那个文件的具体路径
#     if 'img' in dir():
#         pass
#     else:
#         img = g.fileopenbox('open file' + '会导入当前文件夹的全部图片')
#         domain(img)



# -*- coding=utf-8

# SCSD-PY001

# hi-windom/ColorThemeAnalyse

# https://gitee.com/hi-windom/color-theme-analyse

'''
# ---------------------------------
# 创建于    2021-5-18
# 更新于    2021-7-20 02:09:40
# ---------------------------------
# Need help ?  => 694357845@qq.com
# ---------------------------------
# 如果你的 Python 版本大于 3.8.0 ，直接运行本文件，会自动帮你安装依赖
# 建议在 Conda 虚拟环境运行本文件
# ---------------------------------
'''

################################################################
import socket
import time
import os,sys,shutil
from importlib import import_module
root_path = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
SRC_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), '..','src'))
if root_path not in sys.path:
    sys.path.append(root_path)

    # logger = import_module('.logger','lib')
    # logger = logger.myLogging("gitee.com/soltus")
    # executer = import_module('.executer','scp.scripts')

from MMCQsc.scp.lib.logger import *
logger = myLogging("gitee.com/soltus")
from MMCQsc.scp.scripts import executer


from multiprocessing import shared_memory  # required for Python >= 3.8
from concurrent import futures
from subprocess import Popen,PIPE
import shlex

PN = 'easygui'
try:
    import easygui as g
except ImportError:
    logger.critical(f'import "{PN}" could not be resolved')
    logger.info(f"Try to download [{PN}] , Please wait for busy.")
    os.system("pip install easygui -i https://pypi.douban.com/simple/")
    exit()

def get_host_ip():
    """
    查询本机ip地址
    :return: ip
    """
    try:
        ss = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        ss.connect(('8.8.8.8', 80))
        ip = ss.getsockname()[0]
    finally:
        ss.close()

    return ip


def createServer():
    myip = get_host_ip()
    logger.info(SRC_DIR)
    args = shlex.split(f"pyhton -m http.server 5858")
    '''
    cwd 工作目录，设置为正确值以确保 launch ../src/index.html
    executable 参数指定一个要执行的替换程序。这很少需要。当 shell=True， executable 替换 args 指定运行的程序。但是，原始的 args 仍然被传递给程序。大多数程序将被 args 指定的程序作为命令名对待，这可以与实际运行的程序不同。
    '''
    result =  Popen(args, bufsize=0, executable=sys.executable, close_fds=False, shell=False, cwd=SRC_DIR, startupinfo=None, creationflags=0) # shell=False 非常重要
    logger.debug(f"http.server进程 PID: {result.pid}")
    logger.info(f'\n\n\n\t\t本地服务器创建成功：\n\n\t\t{myip}:5858\n\n\t\t（支持局域网访问）\n\n')
    logger.warning("\n\n\t\t[ tip ] : 快捷键 CTR + C 强制结束\n\n")
    result.wait()

def openhtml():
    myip = get_host_ip()
    time.sleep(2)
    logger.info(f'\n\n\n\t\t即将默认浏览器打开：\n\n\t\t{myip}:5858\n\n')
    os.system(f'start http://{myip}:5858')

def mainFunc():
    try:
        shm = shared_memory.SharedMemory(
            name='main_run_share', create=True, size=4096)
    except:
        shm = shared_memory.SharedMemory(name='main_run_share')
        shm.buf[1] = 0
    buf = shm.buf
    # fileopenbox()函数的返回值是你选择的那个文件的具体路径
    if buf[1] > 0:
        pass
    else:
        try:
            img = g.fileopenbox('open file' + '会导入当前文件夹的全部图片')
            if img != None:  # 有传入才处理
                buf[1] = len(img)
                for i in range(2, 10, 1):
                    buf[i] = 0
                executer.domain(img)
                if buf[2] == 1:
                    with futures.ProcessPoolExecutor(max_workers=None) as prolist:
                        prolist.submit(createServer)
                        prolist.submit(openhtml)  # 多进程才能打开
        except BaseException as e:
            if isinstance(e, KeyboardInterrupt):
                os.system('cls')
                logger.info("http.server 进程服务已停止\t原因：用户强制退出")
        finally:
            try:
                os.remove(os.path.join(SRC_DIR + '\\index.js'))
                os.remove(os.path.join(SRC_DIR + '\\index.css'))
                shutil.rmtree(os.path.join(SRC_DIR + '\\finish'))
                shutil.rmtree(os.path.join(SRC_DIR + '\\compress'))
                logger.info('成功删除不重要的自动生成文件')
            except:
                logger.warning('未能删除自动生成文件')
            finally:
                exit()


if __name__ == '__main__':
    import os,sys
    root_path = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
    if root_path not in sys.path:
        sys.path.append(root_path)
    from scp import executable_check
    try:
        result = mainFunc()
    except BaseException as e:
        if isinstance(e, KeyboardInterrupt):
            logger.warning("{}\n\t\t用户强制退出".format(__file__))
    finally:
        exit()


# -*- coding=utf-8

# SCSD-PY001

# hi-windom/ColorThemeAnalyse

# https://gitee.com/hi-windom/color-theme-analyse

'''
# ---------------------------------
# 创建于    2021-7-20
# 更新于    2021-7-20 02:08:57
# ---------------------------------
# Need help ?  => 694357845@qq.com
# ---------------------------------
# 作者很懒，还没想好说些什么
# ---------------------------------
'''

################################################################
import os,sys
from importlib import import_module
import json
root_path = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
if root_path not in sys.path:
    sys.path.append(root_path)
    # logger = import_module('.logger','lib')
    # logger = logger.myLogging("gitee.com/soltus")

from MMCQsc.scp.lib.error_sc import *
from MMCQsc.scp.lib.logger import *
logger = myLogging("gitee.com/soltus")
from MMCQsc.scp.lib.error_sc import *


def fun_version(v1,v2):
# v1 == v2 return 0
# v1 > v2 return 1
# v1 < v2 return -1

    l_1 = v1.split('.')
    l_2 = v2.split('.')
    c = 0
    while True:
        if c == len(l_1) and c == len(l_2):
            return 0
        if len(l_1) == c:
            l_1.append(0)
        if len(l_2) == c:
            l_2.append(0)
        if int(l_1[c]) > int(l_2[c]):
            return 1
        elif int(l_1[c]) < int(l_2[c]):
            return -1
        c += 1


os.system("cls")

file_path = sys.argv[0]
def check_conda():
    if "\\envs\\" in sys.executable:
        conda_exec = os.path.abspath(os.path.join(os.path.dirname(sys.executable), "../..", "Script", "conda.exe"))
        conda_env = sys.executable.split("\\")[-2]
    else:
        conda_exec = os.path.abspath(os.path.join(os.path.dirname(sys.executable), "Script", "conda.exe"))
        conda_env = "base"
    return conda_exec, conda_env


from subprocess import Popen
import shlex

# try:
#     raise error_sc.EnvError('e97304')
# except error_sc.EnvError as e:
#     logger.error("引发异常：" + repr(e))
#     tb = sys.exc_info()[2]
#     if tb:
#         raise error_sc.OA().with_traceback(tb)

def run_in_env(env):
    PY3_VNO = ''
    for i in sys.version_info[:3]:
        PY3_VNO += str(i)
    PY3_VNO = '.'.join(PY3_VNO)
    os.system("cls")
    logger.info("You are using Conda , Activated conda env : '{}' Python {}".format(env, PY3_VNO))
    with os.popen("conda --version") as conda_v:
        if "conda" in conda_v.read():
            logger.debug("\n\n\n\t\t使用当前 Conda 环境继续吗 (y) ？\n\t\t或者重新选择运行环境 (n) ？\n\t\t也可以输入任意字符作为新环境名，将为你自创建一个 Python 3.9.5 的新环境\n\n\t\tProccess ?  [Y/n/*]")
    while True:
        try:
            pick_env = input("main.py:93 >>> ")
        except BaseException as e:
            if isinstance(e, KeyboardInterrupt):
                logger.warning("用户强制退出")
                exit()
        if pick_env in ['Y','y']:
            if sys.version_info.major < 3:
                logger.error(" Can NOT run in Python 2.x ")
                raise EnvError('\n\n\t''This script is only for use with ''Python 3.6 or later\n\n\t https://gitee.com/hi-windomcolor-theme-analyse/ \n\n')
            elif sys.version_info[:3] < (3,6,0):
                logger.error(" Can NOT run in Python < 3.6 ")
                raise EnvError('\n\n\t''This script is only for use with ''Python 3.6 or later\n\n\t https://gitee.com/hi-windomcolor-theme-analyse/ \n\n')
            else:
                return env
        elif pick_env in ['N','n']:
            python = sys.executable.replace(check_conda()[1],pick_env)
            print(python)
            os.system("cls")
            os.system("conda info -e")
            logger.debug("\n\n\n\t\t输入你想激活的 Conda 环境")
            pick_env = input("main.py:109 >>> ")
            logger.debug(f"请在终端执行指令 conda activate {pick_env} 手动激活环境")
            logger.warning("\n\n\t\t[ tip ] : 方向上键 ^ 可调出调出历史指令\n\n")
            exit()
        else:
            os.system("conda deactivate")
            os.system("deactivate")
            os.system("cls")
            python = sys.executable.replace(check_conda()[1],pick_env)
            change_env = file_path.replace('main','change_env')
            try:
                args = shlex.split(f"conda create -n {pick_env} python==3.9.5 -y")
                result = Popen(args, bufsize=0, executable=None, close_fds=False, shell=True, env=None, startupinfo=None, creationflags=0)
                logger.debug(f"创建下载线程 PID: {result.pid}")
                logger.warning("\n\n\t\t[ tip ] : 快捷键 CTR + C 强制结束\n\n")
                result.wait()
            except BaseException as e:
                if isinstance(e, KeyboardInterrupt):
                    logger.warning("用户中止了下载")
                    logger.warning("当前窗口已完成使命，是时候和它告别了")
                    exit()
            #os.system(f"conda create -n {pick_env} python==3.9.5 -y")
            # args = shlex.split(f"conda activate {pick_env}")
            # result = Popen(args, bufsize=0, executable=None, close_fds=False, shell=True, env=None, startupinfo=None, creationflags=0)
            logger.debug(check_conda()[0])
            logger.debug(python)
            logger.debug(file_path)
            logger.debug(f"已创建的环境 : [ {pick_env} ]  请使用创建的环境重新运行\n\n")
            logger.warning("\n\n\t\t[ tip ] : 方向上键 ^ 可调出调出历史指令\n\n")
            exit()
            return pick_env

pick_env = check_conda()[1]
while pick_env:
    env_tmep = pick_env
    pick_env = run_in_env(pick_env)
    if pick_env == env_tmep:
        break

PY3_VNO = ''
for i in sys.version_info[:3]:
        PY3_VNO += str(i)
PY3_VNO = '.'.join(PY3_VNO)
logger.warning("You are using Python {}".format(PY3_VNO))
if fun_version(PY3_VNO,"3.8.0") == -1:
    logger.critical("Required version : Python >= 3.8.0")
    with os.popen("conda --version") as conda_v:
        if "conda" in conda_v.read():
            logger.info("You are using Conda , Press key 'y' to upgrade your Python")
            logger.info("If you want to upgrade later by yourself , use command: conda install python==3.9.5")
            logger.debug("Upgrade your Python to 3.9.5 ?  [Y/*]")
        isupdate = input("main.py:123 >>> ")
    if isupdate not in ['Y','y']:
        exit()
    os.system("cls")
    logger.info("即将开始下载，这取决于你的网络")
    try:
        args = shlex.split(f"conda conda install python==3.9.5 -n {pick_env} -y")
        result = Popen(args, bufsize=0, executable=r"C:\Windows\System32\WindowsPowerShell\v1.0\powershell.exe", close_fds=False, shell=False, env=None, startupinfo=None, creationflags=0)
        logger.debug(f"创建下载线程 PID: {result.pid}")
        logger.warning("\n\n\t\t[ tip ] : 快捷键 CTR + C 强制结束\n\n")
        result.wait()
    except BaseException as e:
        if isinstance(e, KeyboardInterrupt):
            logger.warning("用户中止了下载")
            logger.warning("当前窗口已完成使命，是时候和它告别了")
            result.kill()
    finally:
        if result.returncode:
            logger.error("下载失败，请手动升级 Python 后重试")
        else:
            args = [sys.executable, file_path]
            logger.debug(args)
            logger.debug(f"请在终端执行指令 conda activate {pick_env} 手动激活环境")
            logger.warning("\n\n\t\t[ tip ] : 方向上键 ^ 可调出调出历史指令\n\n")
            exit()
elif fun_version(PY3_VNO,"3.9.5") == -1:
    logger.warning("Recommended version : Python >= 3.9.5  However, it doesn't matter")



# -*- coding=utf-8

# SCSD-PY001

# hi-windom/ColorThemeAnalyse

# https://gitee.com/hi-windom/color-theme-analyse

'''
# ---------------------------------
# 创建于    ???
# 更新于    2021-7-20 02:10:25
# ---------------------------------
# Need help ?  => 694357845@qq.com
# ---------------------------------
# 这是第三方开源项目的一部分源代码
# 这是一个修改过的副本，获取原作者构建的版本请在 GitHub 搜索 [ImageColorTheme]
# ---------------------------------
'''
import sys
import numpy as np
from queue import PriorityQueue as PQueue
from functools import reduce

# PQueue lowest first!
DEBUG = False


class VBox(object):
    """
        The color space is divided up into a set of 3D rectangular regions (called `vboxes`)
    """

    def __init__(self, r1, r2, g1, g2, b1, b2, histo):
        super(VBox, self).__init__()
        self.r1 = r1
        self.r2 = r2
        self.g1 = g1
        self.g2 = g2
        self.b1 = b1
        self.b2 = b2
        self.histo = histo

        ziped = [(r1, r2), (g1, g2), (b1, b2)]
        sides = list(map(lambda t: abs(t[0] - t[1]) + 1, ziped))
        self.vol = reduce(lambda x, y: x*y, sides)
        self.mAxis = sides.index(max(sides))
        self.plane = ziped[:self.mAxis] + ziped[self.mAxis+1:]
        self.npixs = self.population()
        self.priority = self.npixs * -1

    def population(self):
        s = 0
        for r in range(self.r1, self.r2+1):
            for g in range(self.g1, self.g2+1):
                for b in range(self.b1, self.b2+1):
                    s += self.histo[MMCQ.getColorIndex(r, g, b)]
        return int(s)

    def __lt__(self, vbox):  # 实现<操作
        return self.priority < vbox.priority

    def contains(self, r, g, b):
        # real r, g, b here
        pass


class MMCQ(object):
    """
        Modified Median Cut Quantization(MMCQ)
        Leptonica: http://tpgit.github.io/UnOfficialLeptDocs/leptonica/color-quantization.html
    """
    MAX_ITERATIONS = 1000
    SIGBITS = 5

    def __init__(self, pixData, maxColor, fraction=0.85, sigbits=5,file=''):
        """
        @pixData        Image data [[R, G, B], ...]
        @maxColor       Between [2, 100]
        @fraction       Between [0.3, 0.9]
        @sigbits        5 or 6
        """
        super(MMCQ, self).__init__()
        self.pixData = pixData
        if not 2 <= maxColor <= 100:       #源码此处应为[2,256]
            raise AttributeError("maxColor should between [2, 100]!")
        self.maxColor = maxColor
        if not 0.3 <= fraction <= 0.9:
            raise AttributeError("fraction should between [0.3, 0.9]!")
        self.fraction = fraction
        if sigbits != 5 and sigbits != 6:
            raise AttributeError("sigbits should be either 5 or 6!")
        self.SIGBITS = sigbits
        self.rshift = 8 - sigbits
        self.imgPath = file  # 源码没有 [self.imgPath] ，这是为了多线程调试定位添加的
        self.h, self.w, _ = self.pixData.shape

    def getPixHisto(self):
        pixHisto = np.zeros(1 << (3 * self.SIGBITS))
        for y in range(self.h):
            for x in range(self.w):
                r = self.pixData[y, x, 0] >> self.rshift
                g = self.pixData[y, x, 1] >> self.rshift
                b = self.pixData[y, x, 2] >> self.rshift

                pixHisto[self.getColorIndex(r, g, b)] += 1
        return pixHisto

    @classmethod
    def getColorIndex(cls, r, g, b):  # 这里的 [cls] 是被修改的，源码此处应为 [self]
        return (r << (2 * cls.SIGBITS)) + (g << cls.SIGBITS) + b

    def createVbox(self, pixData):
        rmax = np.max(pixData[:, :, 0]) >> self.rshift
        rmin = np.min(pixData[:, :, 0]) >> self.rshift
        gmax = np.max(pixData[:, :, 1]) >> self.rshift
        gmin = np.min(pixData[:, :, 1]) >> self.rshift
        bmax = np.max(pixData[:, :, 2]) >> self.rshift
        bmin = np.min(pixData[:, :, 2]) >> self.rshift

        if DEBUG:
            print("Red range: {0}-{1}".format(rmin, rmax))
            print("Green range: {0}-{1}".format(gmin, gmax))
            print("Blue range: {0}-{1}".format(bmin, bmax))
        return VBox(rmin, rmax, gmin, gmax, bmin, bmax, self.pixHisto)

    def medianCutApply(self, vbox):
        npixs = 0
        if vbox.mAxis == 0:
            # Red axis is largest
            plane = 0
            for r in range(vbox.r1, vbox.r2+1):
                for g in range(vbox.g1, vbox.g2+1):
                    for b in range(vbox.b1, vbox.b2+1):
                        h = vbox.histo[self.getColorIndex(r, g, b)]
                        plane += h
                        npixs += h
                if npixs >= vbox.npixs / 2.:
                    left = r - vbox.r1
                    right = vbox.r2 - r
                    if left >= right:
                        r2 = int(max(vbox.r1, r - 1 - left / 2))
                    else:
                        r2 = int(min(vbox.r2 - 1, r + right / 2))
                    vbox1 = VBox(vbox.r1, r2, vbox.g1, vbox.g2,
                                 vbox.b1, vbox.b2, vbox.histo)
                    vbox2 = VBox(r2+1, vbox.r2, vbox.g1, vbox.g2,
                                 vbox.b1, vbox.b2, vbox.histo)
                    return vbox1, vbox2
        elif vbox.mAxis == 1:
            # Green axis is largest
            for g in range(vbox.g1, vbox.g2+1):
                plane = 0
                for r in range(vbox.r1, vbox.r2+1):
                    for b in range(vbox.b1, vbox.b2+1):
                        h = vbox.histo[self.getColorIndex(r, g, b)]
                        plane += h
                        npixs += h
                if npixs >= vbox.npixs / 2.:
                    left = g - vbox.g1
                    right = vbox.g2 - g
                    if left >= right:
                        g2 = int(max(vbox.g1, g - 1 - left / 2))
                    else:
                        g2 = int(min(vbox.g2 - 1, g + right / 2))
                    vbox1 = VBox(vbox.r1, vbox.r2, vbox.g1, g2,
                                 vbox.b1, vbox.b2, vbox.histo)
                    vbox2 = VBox(vbox.r1, vbox.r2, g2+1, vbox.g2,
                                 vbox.b1, vbox.b2, vbox.histo)
                    return vbox1, vbox2
        else:
            # Blue axis is largest
            for b in range(vbox.b1, vbox.b2+1):
                plane = 0
                for r in range(vbox.r1, vbox.r2+1):
                    for g in range(vbox.b1, vbox.b2+1):
                        h = vbox.histo[self.getColorIndex(r, g, b)]
                        plane += h
                        npixs += h
                if npixs >= vbox.npixs / 2.:
                    left = b - vbox.b1
                    right = vbox.b2 - b
                    if left >= right:
                        b2 = int(max(vbox.b1, b - 1 - left / 2))
                    else:
                        b2 = int(min(vbox.b2 - 1, b + right / 2))
                    vbox1 = VBox(vbox.r1, vbox.r2, vbox.g1,
                                 vbox.g2, vbox.b1, b2, vbox.histo)
                    vbox2 = VBox(vbox.r1, vbox.r2, vbox.g1,
                                 vbox.g2, b2+1, vbox.b2, vbox.histo)
                    return vbox1, vbox2

    def iterCut(self, maxColor, boxQueue, vol=False):
        ncolors = 1
        niters = 0
        while True:
            if ncolors >= maxColor:
                break
            vbox0 = boxQueue.get()[1]  # 这里的 get() 是被修改的，源码此处应为 get_nowait()
            if vbox0.npixs == 0:
                print("Vbox has no pixels")
                boxQueue.put((vbox0.priority, vbox0))
                continue
            try:
                vbox1, vbox2 = self.medianCutApply(vbox0)
            except:
                print(self.imgPath + '  failed to analyse')
                continue

            if vol:
                vbox1.priority *= vbox1.vol
            boxQueue.put((vbox1.priority, vbox1))
            if vbox2 is not None:
                ncolors += 1
                if vol:
                    vbox2.priority *= vbox2.vol
                boxQueue.put((vbox2.priority, vbox2))
            niters += 1
            if niters >= self.MAX_ITERATIONS:
                print("infinite loop; perhaps too few pixels!")
                break
        return boxQueue

    def boxAvgColor(self, vbox):
        ntot = 0
        mult = 1 << self.rshift
        rsum = 0
        gsum = 0
        bsum = 0
        for r in range(vbox.r1, vbox.r2+1):
            for g in range(vbox.g1, vbox.g2+1):
                for b in range(vbox.b1, vbox.b2+1):
                    h = vbox.histo[self.getColorIndex(r, g, b)]
                    ntot += h
                    rsum += int(h * (r + 0.5) * mult)
                    gsum += int(h * (g + 0.5) * mult)
                    bsum += int(h * (b + 0.5) * mult)
        if ntot == 0:
            avgs = map(lambda x: x * mult / 2,
                       [vbox.r1 + vbox.r2 + 1, vbox.g1 + vbox.g2 + 1, vbox.b1 + vbox.b2 + 1])
        else:
            avgs = map(lambda x: x / ntot, [rsum, gsum, bsum])
        return list(map(lambda x: int(x), avgs))

    def quantize(self):
        if self.h * self.w < self.maxColor:
            raise AttributeError(
                "Image({0}x{1}) too small to be quantized".format(self.w, self.h))
        self.pixHisto = self.getPixHisto()

        orgVbox = self.createVbox(self.pixData)
        pOneQueue = PQueue(self.maxColor)
        pOneQueue.put((orgVbox.priority, orgVbox))
        popcolors = int(self.maxColor * self.fraction)

        pOneQueue = self.iterCut(popcolors, pOneQueue)

        boxQueue = PQueue(self.maxColor)
        while not pOneQueue.empty():
            vbox = pOneQueue.get()[1]
            vbox.priority *= vbox.vol
            boxQueue.put((vbox.priority, vbox))
        boxQueue = self.iterCut(self.maxColor - popcolors + 1, boxQueue, True)

        theme = []
        while not boxQueue.empty():
            theme.append(self.boxAvgColor(boxQueue.get()[1]))
        return theme

    """
        empty()
        如果队列是空的，返回 True ，反之返回 False 。 由于多线程或多进程的环境，该状态是不可靠的。

    [这是已知缺陷]

        full()
        如果队列是满的，返回 True ，反之返回 False 。 由于多线程或多进程的环境，该状态是不可靠的。
    """


from enum import Enum,unique
class Error(Exception):
    """Base class for exceptions in this module."""
    pass

class EnvError(Error):
    def __init__(self,id='e00000'):
        self.show(id)

    def show(self,id):
        self.id = id
        self.name = self.get_name(id)
        self.description = self.get_description(id)

    def get_name(self,id):
        return self.__PEE[id].value[0]

    def get_description(self,id):
        return self.__PEE[id].value[1]

    def help(self,id):
        return [id,self.get_name(id),self.get_description(id)]

    def get_all(self,type='dict'):
        result = {}
        for i in self.__PEE:
            result[i.name] = i.value
        if type != 'json':
            return result
        else:
            import json
            jsonStr = json.dumps(result,ensure_ascii=False, sort_keys=True, indent=4, separators=(' ,', ' : \n    '))
            return jsonStr


    # @unique
    class __PEE(Enum):
        e00000 = ('未知的错误','这是开发者定义的异常，但未提供有关异常的任何帮助信息')
        e81630 = ('',' Can NOT run in Python 2.x ')
        e57453 = ('',' Can NOT run in Python < 3.6 ')
        e97304 = ('','\n\n\t''This script is only for use with ''Python 3.6 or later\n\n\t https://gitee.com/hi-windom/color-theme-analyse/ \n\n')
        e92577 = ('','')
        e16651 = ('','')
        e00324 = ('','')
        e92532 = ('','')
        e13599 = ('','')
        e30056 = ('','')
        e13877 = ('','')
        e01874 = ('','')
        e83112 = ('','')
        e51957 = ('','')
        e34931 = ('','')
        e12154 = ('','')
        e32709 = ('','')
        e34688 = ('','')
        e79646 = ('','')
        e80476 = ('','')
        e26224 = ('','')
        e60388 = ('','')
        e75028 = ('','')
        e10496 = ('','')
        e61843 = ('','')


import setuptools
'''
官方推荐使用静态的 setup.cfg 但动态的 setup.py 对我们来说更熟悉，学习成本低，两个文件也可以共存
MANIFEST.in 需要放在和 setup.py 同级的顶级目录下，setuptools 会自动读取该文件，需要注意 MANIFEST.in 指令是按顺序执行的，因此 exclude 要放在 include 后面
建议 MANIFEST.in 只用于构建 tar.gz 而不用于 whl ，即 setup.py 设置 [include_package_data=False]，至少在熟练掌握构建前应当这样做

//关于setuptools：setuptools 是 distutils 增强版，不包括在标准库中
//关于包格式：egg 包是过时的，whl 包是新的标准
//关于打包：python -m build 默认帮你生成了 dist/*.tar.gz 和 dist/*.whl ，更多命令使用 python setup.py --help-commands 查看
//关于上传：用 upload 命令上传包已经过时（不安全），官方提供了 twine 工具专门用来与 PyPI 交互。
项目成熟之前，应当使用 twine upload dist/* --verbose --repository testpypi
testpypi 的数据库会被定期修剪，因此可以放心上传
//关于版本号：Python 的软件分发工具还支持 local version identifier 可用于标识不打算发布的本地开发构建，本地版本标识符采用以下形式 <public version identifier>+<local version label> 例如：

        1.2.0.dev1+hg.5.b11e5e6f0b0b  # 5th VCS commmit since 1.2.0.dev1 release
        1.2.1+fedora.4                # Package with downstream Fedora patches applied

另请注意，使用 setuptools_scm 控制版本后，使用了本地版本标识符是无法上传到 PyPi 的，因此 local_scheme = "no-local-version" 在 pyproject.toml
修改 Release 版本号需要使用 Git 打上版本号标签，在熟悉之前应当使用 x.x.x 形式的标签（例如 1.0.2 ）
如果不熟悉 Git 命令行操作，可以使用软件 Sourcetree 直观的提交和打标签。如果没有标签，你生成的包将始终为 0.1.dev*
建议的版本号规则：
模块的版本号采用X.Y.Z的格式，
1、修复bug，小改动，增加z。
2、增加新特性，可向后兼容，增加y
3、有很大的改动，无法向下兼容,增加x
'''

EXAMPLE_EXT = setuptools.Extension(
    name='_test',
    sources=[
        'src/test_wrap.cxx',
        'src/test.cpp',
    ],
)


with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    ext_modules=[EXAMPLE_EXT],
    name="color-theme-analyse",
    setup_requires=['setuptools_scm'], # 指定运行 setup.py 文件本身所依赖的包
    use_scm_version=True, # .gitignore 应与 setup.py 在同一文件夹 更多信息参考 https://pypi.org/project/setuptools-scm/
    # version="0.0.5", # 默认的手动指定版本
    author="Soltus",
    author_email="694357845@qq.com",
    description="SCSD-PY001 info: This is a simple demo of pictures color theme batch analysis use MMCQ with Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitee.com/hi-windom/color-theme-analyse",
    project_urls={
        "Bug Tracker": "https://gitee.com/hi-windom/color-theme-analyse/issues",
    },
    classifiers=[
        # 提供分类项目的分类器列表。有关完整的列表，请参阅https://pypi.org/classifiers/
        # 尽管分类器列表可以声明项目支持的Python版本，但此信息仅用于搜索和浏览Pypi上的项目，而不用于安装项目。
        # 要实际限制项目可以安装的Python版本，请使用 python_requires
        "Development Status :: 4 - Beta",
        "Environment :: Win32 (MS Windows)",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Programming Language :: JavaScript",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Microsoft :: Windows :: Windows 10",
        "Natural Language :: Chinese (Simplified)",
        "Natural Language :: English",
        "Topic :: Multimedia :: Graphics"
    ],
    packages=setuptools.find_packages("src"),
    package_dir={"": "src"},
    exclude_package_data={},
    package_data={
        '':['*.json'],
    }, # 数据文件包含在包的子目录中,也就是有__init__.py 的文件夹中
    data_files=[
        ('lib/site-packages/MMCQsc/src',['src/MMCQsc/src/index.html']),
        ('lib/site-packages/MMCQsc/src',['src/MMCQsc/src/gitee.svg']),
        ('lib/site-packages/MMCQsc/src/_css',['src/MMCQsc/src/_css/base.css','src/MMCQsc/src/_css/animate.css','src/MMCQsc/src/_css/sweet-alert.css']),
        ('lib/site-packages/MMCQsc/src/_js',['src/MMCQsc/src/_js/base.js','src/MMCQsc/src/_js/babel.min_5.8.23.js','src/MMCQsc/src/_js/react-dom.development.js','src/MMCQsc/src/_js/react.development.js','src/MMCQsc/src/_js/sweet-alert.js','src/MMCQsc/src/_js/wow.min.js','src/MMCQsc/src/_js/wow.min2.js']),
    ], # 不在包内的数据文件，格式为(安装目录，文件目录)，注意都是相对路径
    include_package_data=False, # !important
    # entry_points 可以自动将模块入口生成为跨平台的可执行文件(windows平台生成.exe)，也可用于开发插件，如果不了解不要乱写
    # 插件开发以 jupyter-lab 的中文扩展包为例：
    # entry_points={"jupyterlab.languagepack":["zh_CN = jupyterlab_Chinese_SC"],}
    # 注意 jupyterlab_Chinese_SC 并非官方使用的原名，仅供参考
    # 可执行文件以 'console_scripts':['RunMMCQsc = MMCQsc.scp.main:mainFunc'] 为例：
    # console_scripts 是 Python 定义的，可以理解为生成控制台程序脚本
    # RunMMCQsc 为文件名，表示生成 RunMMCQsc.exe 文件，pip install 后该文件就躺在 python.exe 所在文件夹的 Scripts 子文件夹里面
    # MMCQsc.scp.main:mainFunc 只需要知道 mainFunc 是 MMCQsc/scp/main.py 里的一个函数
    entry_points={'console_scripts':['RunMMCQsc = MMCQsc.scp.main:mainFunc']},
    # 手动添加脚本。虽然 scripts 关键字用于指向预先制作好的脚本进行安装，建议使用实现跨平台兼容性的方法 console_scripts 入口点(entry_points)
    scripts=['src/MMCQsc.cmd'],
    license="MIT",
    platforms=['Windows'],
    python_requires=">=3.8.0,<=3.10.0",
    # 关于 install_requires， 有以下五种常用的表示方法：(指定包名是必须的，而版本控制与可选依赖，则是高级形式。 这不仅仅是install_requires的形式，而是对setup.py的所有require都适用)
    # 'argparse'，只包含包名。 这种形式只检查包的存在性，不检查版本。 方便，但不利于控制风险，更新时还会遇到依赖冲突问题。
    # 'setuptools==38.2.4' 指定版本。确保了开发、测试与部署的版本一致，不会出现意外。 缺点是不利于更新，每次更新都需要改动代码。
    # 'docutils >= 0.3'，这是比较常用的形式。 当对某个库比较信任时，这种形式可以自动保持版本为最新。
    # 'Django >= 1.11, != 1.11.1, <= 2'，这是比较复杂的形式。保证了Django的大版本在1.11和2之间，也即1.11.x；并且，排除了已知有问题的版本1.11.1（仅举例）。 对于一些大型、复杂的库，这种形式是最合适的。
    # 'requests[security, socks] >= 2.18.4'，这是包含了额外的可选依赖的形式。 正常安装requests会自动安装它的install_requires中指定的依赖，而不会安装security和socks这两组依赖（这两组依赖是定义在它的extras_require中）。
    install_requires=[
        'easygui>=0.1',
        'opencv-python>=4.0',
        'numpy>=1.21',
        'pillow>=8.3',
        'rich>=0.1',
    ],
    # install_requires 在安装模块时会自动安装依赖包
    # extras_require 仅表示该模块依赖这些包，但不是必须的，需要用户手动安装。被依赖使用时，可以用类似'requests[download, advanced]'的形式来指定
    # extras_require 需要一个 dict ，其中按（自定义的）功能名称进行分组，每组一个 list
    extras_require={
        'download':['pipx>=0.1'],
        'advanced':['pyinstaller>=4.3; platform_system == "Windows"'],
    },
    # dependency_links 已被弃用，因此使用 testPyPi 测试时，一些依赖项会无法下载（ https://test-files.pythonhosted.org 上没有），需要提前安装好依赖，然后 pip 后面加参数 --no-deps 无依赖下载
    dependency_links=[
        'https://pypi.douban.com/simple',
        'https://pypi.org/simple',
    ],
)